<?php

/**
 * @file
 * Contains \Drupal\webfactory_master\Plugin\Channel\Source\Bundle.
 */

namespace Drupal\webfactory_master\Plugin\Channel\Source;

use Drupal\Core\Form\FormStateInterface;
use Drupal\webfactory_master\Plugin\Channel\ChannelSourceBase;

/**
 * Allow to create a channel that expose bundles.
 *
 * @ChannelSource(
 *   id = "bundle",
 *   label = @Translation("Bundle")
 * )
 */
class Bundle extends ChannelSourceBase {

  /**
   * Return list of entities according to configuration.
   */
  public function entities() {
    $entities = array();
    $types = array();
    $entity_type = $this->channelEntity->get('entity_type');

    if (isset($this->settings['bundle'])) {
      $bundles = $this->settings['bundle'];
      foreach ($bundles as $bundle) {
        $types[] = $bundle;
      }
    }

    $bundle_prop = $this->getBundleTypeProperty($entity_type);

    $query = \Drupal::entityQuery($entity_type);
    if (!empty($types)) {
      $query->condition($bundle_prop, $types);
    }
    $entities_id = $query->execute();

    if (!empty($entities_id)) {
      $entities = \Drupal::entityManager()->getStorage($entity_type)->loadMultiple($entities_id);
    }

    return $entities;
  }

  /**
   * Return plugin settings form.
   *
   * @param array $form
   *   The form element.
   * @param FormStateInterface $form_state
   *   The form state element.
   *
   * @return mixed
   *   The form.
   */
  public function getSettingsForm(array $form, FormStateInterface $form_state) {
    $bundle_values = [];

    foreach ($this->settings['bundle'] as $bundle) {
      $bundle_values[] = $bundle;
    }

    $entity_type = $this->channelEntity->get('entity_type');

    $bundle_form['bundle'] = [
      '#type' => 'select',
      '#title' => $this->t('Bundles'),
      '#multiple' => TRUE,
      '#default_value' => $bundle_values,
      '#prefix' => '<div id="edit-bundle-list-wrapper">',
      '#suffix' => '</div>',
    ];

    $bundle_form['bundle']['#options'] = $this->getBundleList($entity_type);

    return $bundle_form;
  }

  /**
   * Retrieve settings to store in channel entity.
   *
   * @param array $form
   *   The form element.
   * @param FormStateInterface $form_state
   *   The form state element.
   *
   * @return array
   *   get settings list.
   */
  public function getSettings(array $form, FormStateInterface $form_state) {
    $settings = ['bundle' => []];

    $entities = $form_state->getValue('bundle');
    foreach ($entities as $bundle_id => $selected) {
      if ($selected != '0') {
        $settings['bundle'][] = $bundle_id;
      }
    }

    return $settings;
  }

  /**
   * Helper to retrieve bundle from given entities.
   *
   * @param string $entity_type
   *   Entity type.
   *
   * @return array
   *   Bundle optuions.
   */
  protected function getBundleList($entity_type) {
    $bundles_options = [];
    $bundles = \Drupal::entityManager()->getBundleInfo($entity_type);
    foreach ($bundles as $bundle_id => $bundle) {
      $bundles_options[$bundle_id] = $bundle['label'];
    }

    return $bundles_options;
  }

  /**
   * Get bundle type property.
   *
   * @param string $entity_type
   *   The entity type.
   *
   * @return null
   *   The bundle type property.
   */
  protected function getBundleTypeProperty($entity_type) {
    $bundle_prop = NULL;
    $definitions = \Drupal::entityManager()->getDefinitions();
    if (isset($definitions[$entity_type])) {
      $entity_keys = $definitions[$entity_type]->get('entity_keys');
      if (isset($entity_keys['bundle'])) {
        $bundle_prop = $entity_keys['bundle'];
      }
    }
    return $bundle_prop;
  }

}
