<?php

/**
 * @file
 * Contains Drupal\webfactory_master\src\Plugin\rest\resource\ChannelResource.
 */

namespace Drupal\webfactory_master\Plugin\rest\resource;

use Drupal\rest\Plugin\ResourceBase;
use Drupal\rest\ResourceResponse;
use Drupal\webfactory_master\Entity\ChannelEntity;
use Drupal\webfactory_master\Entity\SatelliteEntity;
use Drupal\webfactory_master\Plugin\ChannelSourcePluginManager;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Component\Routing\Route;

/**
 * Channel ressource.
 *
 * @RestResource(
 *   id = "webfactory_master:channel",
 *   label = "Channel",
 *   uri_paths = {
 *     "canonical" = "/webfactory_master/channel",
 *   }
 * )
 */
class ChannelResource extends ResourceBase {

  /**
   * Constructor.
   *
   * @param array $configuration
   *   Configuration.
   * @param string $plugin_id
   *   Plugin id.
   * @param mixed $plugin_definition
   *   Plugin definition.
   * @param array $serializer_formats
   *   Serializer format.
   * @param ChannelSourcePluginManager $channel_source_manager
   *   Channel source manager.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, array $serializer_formats, LoggerInterface $logger, ChannelSourcePluginManager $channel_source_manager) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $serializer_formats, $logger);

    $this->channelSourceManager = $channel_source_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->getParameter('serializer.formats'),
      $container->get('logger.factory')->get('rest'),
      $container->get('plugin.manager.webfactory_master.channel')
    );
  }

  /**
   * Responds to GET requests.
   *
   * Returns a watchdog log entry for the specified ID.
   *
   * @param int $uuid
   *   The ID of the watchdog log entry.
   * @param int $channel_id
   *   The channel id.
   *
   * @return ResourceResponse
   *   The response containing the log entry.
   */
  public function get($uuid = NULL, $channel_id = NULL) {
    $channel_entity = ChannelEntity::load($channel_id);
    $entity_type = $channel_entity->get('entity_type');
    $channel_source = $channel_entity->get('source');
    $settings = $channel_entity->get('settings');

    $entities = [];

    if ($uuid !== NULL) {
      $entities[] = \Drupal::entityManager()->loadEntityByUuid($entity_type, $uuid);
    }
    else {
      $channel = $this->channelSourceManager->createInstance($channel_source);

      $channel->setConfiguration($channel_entity, $settings);
      $entities = $channel->entities();
    }

    if (!empty($entities)) {
      return new ResourceResponse($this->prepareResponse(
        $entity_type,
        $entities
      ));
    }

    throw new HttpException(t('No valid channel ID was provided'));
  }

  /**
   * Option request.
   *
   * @param int $satellite_id
   *   Satellite entity id.
   *
   * @return ResourceResponse
   *   Request response.
   */
  public function options($satellite_id) {
    \Drupal::logger('webfactory')->debug(serialize(func_get_args()));

    $satellite_entity = SatelliteEntity::load($satellite_id);
    $channels = $satellite_entity->get('channels');

    $channel_entities = ChannelEntity::loadMultiple($channels);
    return new ResourceResponse($channel_entities);
  }

  /**
   * Prepare response.
   *
   * @param string $entity_type
   *   The entity type.
   * @param array $entities
   *   List of entities.
   *
   * @return array
   *   Response.
   */
  protected function prepareResponse($entity_type, array $entities) {
    $response = [
      'entity_type' => $entity_type,
      'entities' => [],
    ];

    foreach ($entities as $entity) {
      $bundle = $entity->bundle();
      if (!isset($response['entities'][$bundle])) {
        $response['entities'][$bundle] = [$entity];
      }
      else {
        $response['entities'][$bundle][] = $entity;
      }
    }
    return $response;
  }

  /**
   * Provides predefined HTTP request methods.
   *
   * Plugins can override this method to provide additional custom request
   * methods.
   *
   * @return array
   *   The list of allowed HTTP request method strings.
   */
  protected function requestMethods() {
    return array(
      'GET',
      'OPTIONS',
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getBaseRoute($canonical_path, $method) {
    $lower_method = strtolower($method);

    $default = [
      '_controller' => 'Drupal\rest\RequestHandler::handle',
      // Pass the resource plugin ID along as default property.
      '_plugin' => $this->pluginId,
    ];

    $parameters = [];
    switch ($method) {
      case 'GET':
        $canonical_path .= '/{channel_id}/{uuid}';
        $default['uuid'] = NULL;
        break;

      case 'OPTIONS':
        $canonical_path .= '/{satellite_id}';
        break;
    }

    $route = new Route($canonical_path, $default,
      array(
        '_permission' => "restful $lower_method $this->pluginId",
      ),
      $parameters,
      '',
      array(),
      // The HTTP method is a requirement for this route.
      array($method)
    );

    return $route;
  }

}
