<?php

/**
 * @file
 * Contains Drupal\webfactory_slave\Form\TableEntitiesForm.
 */

namespace Drupal\webfactory_slave\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\webfactory_slave\EntitySyncWrapper;
use Drupal\webfactory_slave\Services\EntitySynchronizer;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the table entities form.
 *
 * @package Drupal\webfactory_slave\Form
 */
class TableEntitiesForm extends FormBase {

  /**
   * The rest services.
   *
   * @var mixed
   */
  private $entitySync;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('webfactory_slave.services.entity_synchronizer')
    );
  }

  /**
   * TableEntitiesForm constructor.
   *
   * @param \Drupal\webfactory_slave\Services\EntitySynchronizer $entity_sync
   *    Entity synchroniser.
   */
  public function __construct(EntitySynchronizer $entity_sync) {
    $this->entitySync = $entity_sync;
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'webfactory_slave_tableentities_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $type_filter  = \Drupal::request()->get('type');
    $sync_filter  = \Drupal::request()->get('sync');
    $title_filter = \Drupal::request()->get('title');

    $data = $this->entitySync->getData($type_filter);

    $header_form = array(
      'title' => array('data' => t('title'), 'sort' => 'asc'),
      'type' => array('data' => t('content type'), 'sort' => 'asc'),
      'synchronized' => array('data' => t('Synchronized')),
      'status' => array('data' => t('Status')),
    );

    $form['table'] = array(
      '#type' => 'table',
      '#tableselect' => TRUE,
      '#empty' => t('No content available.'),
      '#header' => $header_form,
    );

    $new_items = array();

    $entity_type = $data['entity_type'];
    foreach ($data['entities'] as $bundle => $entities) {
      foreach ($entities as $entity_id => $entity_summary) {
        $entity_wrapper = new EntitySyncWrapper($entity_type, $entity_summary);

        $uuid  = $entity_wrapper->getUuid();
        $title = $entity_wrapper->getTitle();
        $status = $entity_wrapper->getStatus();
        $local_entity = $entity_wrapper->getLocalEntity();

        $status_label = t('New content');
        if ($status == EntitySyncWrapper::NEEDS_UPDATE) {
          $status_label = t('Needs update');
        }
        elseif ($status == EntitySyncWrapper::UPDATED) {
          $status_label = t('Updated');
        }

        $entity_status = [
          '#theme' => 'entity_status',
          '#entity' => [
            'status' => $status,
            'status_label' => $status_label,
          ],
        ];

        if (empty($title_filter) || strpos($title, $title_filter) !== FALSE) {
          $url = 'entity.' . $entity_type . '.canonical';
          $link = $title;

          if ($local_entity != NULL) {
            $link = \Drupal::l($title, new Url($url, array($entity_type => $local_entity->nid->value)));
          }

          $form['table'][$uuid]['title'] = array(
            '#markup' => $link,
          );
          $form['table'][$uuid]['type'] = array(
            '#markup' => $bundle,
          );

          $form['table'][$uuid]['synchronized'] = $entity_status;

          $form['table'][$uuid]['status'] = array(
            '#markup' => '',
          );

          // Filter options after construct.
          switch ($sync_filter) {
            case 'new':
              if (!isset($new_items[$uuid])) {
                unset($form['table'][$uuid]);
              }
              break;

            case 'need':
              if (isset($new_items[$uuid]) || empty($markup)) {
                unset($form['table'][$uuid]);
              }
              break;

            case 'updated':
              if (!empty($markup)) {
                unset($form['table'][$uuid]);
              }
              break;

            default:
          }
        }
      }
    }

    $form['new_item'] = array(
      '#type'  => 'hidden',
      '#value' => $new_items,
    );

    $form['selected'] = array(
      '#type' => 'select',
      '#title' => t('Action'),
      '#options' => array(
        'synchronize' => t('Synchronize'),
        'duplicate' => t('Duplicate'),
      ),
      '#description' => t('Set the action.'),
    );

    $form['actions'] = array('#type' => 'actions');
    $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Update selected contents'),
      '#tableselect' => TRUE,
    );

    $form['pager'] = array('#theme' => 'pager');

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $type_filter  = \Drupal::request()->get('type');

    $values     = $form_state->getValues();
    $action     = $form_state->getValue('selected');
    $build_info = $form_state->getBuildInfo();

    // @todo find better approach to get the entity type
    $entity_type = $build_info['args'][0];

    if ($action == 'synchronize') {
      $count = 0;

      foreach ($values['table'] as $uuid => $is_checked) {
        if ($is_checked != 0) {
          $this->entitySync->save($type_filter, $uuid);
          ++$count;
        }

      }

      drupal_set_message(t('@count contents has been updated', array('@count' => $count)));
    }
    elseif ($action == 'duplicate') {
      $count = 0;
      foreach ($values['table'] as $uuid => $is_checked) {
        if ($is_checked) {
          $entity = \Drupal::entityManager()->loadEntityByUuid($entity_type, $uuid);
          $type_field = $entity->get('type')->getValue();
          $new_entities_values = array(
            'type' => $type_field['0']['target_id'],
            'title' => $entity->get('title'),
            'body' => $entity->get('body'),
          );
          $new_entity = entity_create($entity_type, $new_entities_values);
          $new_entity->save();

          ++$count;
        }
      }

      drupal_set_message(t('@count contents has been duplicated', array('@count' => $count)));
    }
  }

}
