<?php

/**
 * @file
 * Contains Drupal\webfactory_slave\Services\EntitySynchronizer.
 */

namespace Drupal\webfactory_slave\Services;

use Drupal\webfactory_slave\MasterClientInterface;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides Entity synchronization from master site.
 *
 * @package Drupal\webfactory_slave\Services
 */
class EntitySynchronizer implements ContainerInjectionInterface {

  /**
   * Master client instance.
   *
   * @var \Drupal\webfactory_slave\MasterClientInterface
   */
  protected $masterClient;

  /**
   * Channels available for current satellite.
   *
   * @var array of channels retrieved from master
   */
  protected $channels;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('webfactory_slave.services.rest')
    );
  }

  /**
   * Create a new instance of EntitySynchronizer.
   *
   * @param \Drupal\webfactory_slave\MasterClientInterface $client
   *    The master client to use.
   */
  public function __construct(MasterClientInterface $client) {
    $this->masterClient = $client;

    // Init available channels.
    $this->initChannels();
  }

  /**
   * Return channels retrieved from master.
   *
   * @return array
   *    Satellite channels.
   */
  public function getChannels() {
    return $this->channels;
  }

  /**
   * Retrieve entities from given channel id.
   *
   * @param string $channel_id
   *    The channel id.
   *
   * @return array
   *    Remote entities.
   */
  public function getData($channel_id) {
    if ($channel_id == NULL && !empty($this->channels)) {
      $channel = current($this->channels);
      $channel_id = $channel->id;
    }

    return $this->masterClient->getEntitiesData($channel_id);
  }

  /**
   * Create or update local entity according to given uuid.
   *
   * @param string $channel_id
   *    Channel Id used.
   * @param string $uuid
   *    Uuid of entity.
   */
  public function save($channel_id, $uuid) {
    if ($channel_id == NULL && !empty($this->channels)) {
      $channel = current($this->channels);
      $channel_id = $channel->id;
    }

    $response = $this->masterClient->getEntitiesData($channel_id, $uuid);

    $entity_type = $response['entity_type'];
    foreach ($response['entities'] as $bundle => $entities) {
      foreach ($entities as $entity) {
        $bundle_prop = $this->getBundleTypeProperty($entity_type);

        $local_entity = \Drupal::entityManager()->loadEntityByUuid($entity_type, $uuid);

        // Just synchronised existing entity.
        if ($local_entity) {
          $field_definitions = \Drupal::entityManager()->getFieldDefinitions($entity_type, $bundle);
          foreach ($field_definitions as $field_name => $field) {
            if (isset($entity[$field_name]) && !in_array($field_name, ['nid', 'vid'])) {
              $local_entity->set($field_name, $entity[$field_name]);
            }
          }
          $local_entity->save();
        }
        else {
          // Create a copy.
          $entity['uuid'] = $uuid;
          $entity[$bundle_prop] = $bundle;

          $new_entity = \Drupal::entityManager()
            ->getStorage($entity_type)
            ->create($entity);

          $new_entity->save();
        }
      }
    }
  }

  /**
   * Helper that extract bundle property from given entity type.
   *
   * @param string $entity_type
   *    Entity type.
   *
   * @return string
   *    Bundle property.
   */
  protected function getBundleTypeProperty($entity_type) {
    $bundle_prop = NULL;
    $definitions = \Drupal::entityManager()->getDefinitions();
    if (isset($definitions[$entity_type])) {
      $entity_keys = $definitions[$entity_type]->get('entity_keys');
      if (isset($entity_keys['bundle'])) {
        $bundle_prop = $entity_keys['bundle'];
      }
    }
    return $bundle_prop;
  }

  /**
   * Retrieves channels from master and save it into cache (if not empty).
   */
  protected function initChannels() {
    $cid = 'webfactory_slave:channels';

    $data = NULL;
    $channels = [];
    if ($cache = \Drupal::cache()->get($cid)) {
      $channels = $cache->data;
    }
    else {
      $conf_webfactory_slave = \Drupal::configFactory()->getEditable('webfactory_slave.settings');
      $sat_id = $conf_webfactory_slave->get('id');
      $channels = $this->masterClient->getChannelsData($sat_id);

      if (!empty($channels)) {
        \Drupal::cache()->set($cid, $channels);
      }
    }

    $this->channels = $channels;
  }

}
