<?php

/**
 * @file
 * Contains Drupal\webfactory_slave\Services\MasterRestClient.
 */

namespace Drupal\webfactory_slave\Services;

use Drupal\webfactory_slave\MasterClientInterface;
use GuzzleHttp\Client;
use GuzzleHttp\Cookie\CookieJar;
use GuzzleHttp\Exception\RequestException;

/**
 * Webfactory Master REST Client.
 *
 * @package Drupal\webfactory_slave\Services
 */
class MasterRestClient implements MasterClientInterface {

  /**
   * Webfactory slave settings.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  private $config;

  /**
   * HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  private $httpClient;

  /**
   * Drupal user auth cookie.
   *
   * @var \GuzzleHttp\Cookie\CookieJar
   */
  private $cookie;

  /**
   * {@inheritdoc}
   */
  public function __construct() {
    $this->config = \Drupal::config('webfactory_slave.settings');
    $this->createSession();
  }

  /**
   * Retrieve all channel available for given satellite.
   *
   * @param string $satellite_id
   *    Satellite ID.
   *
   * @return array
   *    Channels available for given satellite.
   */
  public function getChannelsData($satellite_id) {
    $token = $this->httpClient->get('/rest/session/token', [
      'cookies' => $this->cookie,
    ])->getBody(TRUE);

    $token = $token->__toString();

    $url = '/webfactory_master/channel/' . $satellite_id;
    $response = $this->httpClient->options($url, [
      'cookies' => $this->cookie,
      'headers' => [
        'Accept' => 'application/json',
        'Content-type' => 'application/hal+json',
        'X-CSRF-Token' => $token,
      ],
      'query' => [
        '_format' => 'json',
      ],
    ]);
    $data = json_decode($response->getBody());

    if ($data == NULL) {
      $data = [];
    }
    else {
      $data = (array) $data;
    }

    return $data;
  }

  /**
   * Retrieves entities from master according to given channel id.
   *
   * You can specify an uuid to fetch only one entity.
   *
   * @param string $channel_id
   *    Channel ID.
   * @param string $uuid
   *    Entity uuid to fetch.
   *
   * @return array|mixed
   *    Return a list of entities or a full entity.
   */
  public function getEntitiesData($channel_id, $uuid = NULL) {
    $data = [];
    $url = '/webfactory_master/channel/' . $channel_id;

    if ($uuid !== NULL) {
      $url .= '/' . $uuid;
    }

    try {
      $response = $this->httpClient->get($url, [
        'cookies' => $this->cookie,
        'headers' => [
          'Accept' => 'application/hal+json',
          'Content-type' => 'application/hal+json',
        ],
        'query' => [
          '_format' => 'hal_json',
        ],
      ]);
      $data = json_decode($response->getBody(), TRUE);
    }
    catch (RequestException $e) {

    }

    return $data;
  }

  /**
   * Initialize a session with the master.
   */
  protected function createSession() {
    $master_ip = $this->getConfig('master_ip');
    $auth = $this->getConfig('authentificate');
    if (!empty($master_ip) && !empty($auth)) {

      $this->httpClient = new Client([
        'base_uri' => 'http://' . $master_ip,
        'cookies' => TRUE,
        'allow_redirects' => TRUE,
      ]);

      // With self sign certificate only.
      /* $this->httpClient->setDefaultOption('verify', FALSE); */

      $this->auth($auth['username'], $auth['password']);
    }
  }

  /**
   * Manage authentification to the master.
   *
   * @param string $name
   *    Drupal user login.
   * @param string $password
   *    Drupal user password.
   */
  protected function auth($name, $password) {
    $this->cookie = new CookieJar();
    $response = $this->httpClient->post('/user/login', [
      "form_params" => [
        "name" => $name,
        "pass" => $password,
        'form_id' => 'user_login_form',
      ],
      'cookies' => $this->cookie,
    ]);
  }

  /**
   * Helper to ease config reading.
   *
   * @param string $key
   *    Config entry key.
   *
   * @return mixed value
   *    Value of entry key.
   */
  protected function getConfig($key) {
    return $this->config->get($key);
  }

}
